<?php

declare(strict_types=1);

namespace Drupal\trinion_client_bank\Controller;

use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\RedirectCommand;
use Drupal\Core\Controller\ControllerBase;
use Drupal\node\Entity\Node;

/**
 * Returns responses for Trinion client-bank routes.
 */
final class SozdaniePlatezhaController extends ControllerBase {

  /**
   * Builds the response.
   */
  public function __invoke(Node $node) {
    $response = new AjaxResponse();
    $company = $node->get('field_tl_kompaniya')->first();
    if (empty($company)) {
      \Drupal::messenger()->addError(t('Company is not set'));
      $response->addCommand(new RedirectCommand('/node/' . $node->id()));
    }
    else {
      $company_type = $company->entity->get('field_tl_tip_kompanii')->getString();
      if (empty($company_type)) {
        \Drupal::messenger()->addError(t('Company type is undefined'));
        $response->addCommand(new RedirectCommand('/node/' . $node->id()));
      }
      elseif ($platezh_id = $node->get('field_tcb_platezh')->getString()) {
        \Drupal::messenger()->addMessage(t('Payments already exists.'));
        $response->addCommand(new RedirectCommand('/node/' . $platezh_id));
      }
      else {
        $uid = \Drupal::currentUser()->id();
        if ($company_type == 'Vendor') {
          $node_data = [
            'type' => 'otpravlennyy_platezh',
            'title' => \Drupal::service('trinion_tp.helper')->getNextDocumentNumber('otpravlennyy_platezh'),
            'field_tp_poluchatel' => $company->entity->id(),
          ];
          \Drupal::messenger()->addMessage(t('Payment made has created.'));
        }
        else {
          $node_data = [
            'type' => 'poluchennyy_platezh',
            'title' => \Drupal::service('trinion_tp.helper')->getNextDocumentNumber('poluchennyy_platezh'),
            'field_tp_platelschik' => $company->entity->id(),
            'field_tp_platezhnaya_sistema' => \Drupal::config('trinion_tp.settings')->get('bankovskiy_perevod_tid'),
          ];
          \Drupal::messenger()->addMessage(t('Payments received has created.'));
        }
        $node_data['uid'] = $uid;
        $node_data['status'] = 1;
        $node_data['field_tl_valyuta'] = 'rub';
        $node_data['field_tp_otvetstvennyy'] = $uid;
        $node_data['field_tp_data'] = $node->get('field_tp_data')->getString();
        $node_data['field_tp_itogo'] = $node->get('field_tp_itogo')->getString();
        $node_data['field_tp_organizaciya'] = $node->get('field_tp_organizaciya')->getString();
        $platezh = Node::create($node_data);
        $platezh->save();
        $platezh_id = $platezh->id();
        $node->field_tcb_platezh = $platezh_id;
        $node->save();
        $response->addCommand(new RedirectCommand('/node/' . $platezh_id));
      }
    }

    return $response;
//    dump($node->get('field_tl_kompaniya')->first());exit;
//    return $build;
  }

}
