<?php

namespace Drupal\trinion_client_bank\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\field\Entity\FieldStorageConfig;
use Drupal\file\Entity\File;
use Drupal\node\Entity\Node;
use Drupal\taxonomy\Entity\Term;

/**
 * Bulk import Client-bank payments Form
 */
class ClientBankImportPaymentsForm extends FormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'zoo_mod_client_bank_import_payments';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['file'] = [
      '#type' => 'managed_file',
      '#title' => t('File'),
      '#required' => TRUE,
      '#upload_validators' => [
        'file_validate_extensions' => ['txt'],
      ],
    ];

    $form['actions'] = [
      '#type' => 'actions',
    ];
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => t('Upload'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $uid = \Drupal::currentUser()->id();
    $file = $form_state->getValue('file');
    if (empty($file[0]))
      return;
    $file = File::load($file[0]);

    $file_path = $file->get('uri')->getString();
    $str = file_get_contents($file_path);
    $str = iconv('Windows-1251', 'UTF-8', $str);
    preg_match_all('/СекцияДокумент=.*?КонецДокумента/s', $str, $matches);
    $count_created = 0;
    foreach ($matches[0] as $match) {
      $lines = explode("\n", $match);
      $data = ['other' => ''];
      foreach ($lines as $line) {
        if ($line == 'КонецДокумента')
          break;
        list($param, $value) = explode('=', $line);
        $value = trim($value);
        switch ($param) {
          case 'СекцияДокумент':
            $data['client_bank_type_doc'] = $value;
            break;
          case 'Номер':
            $data['nomer'] = $value;
            break;
          case 'Дата':
            $data['document_date'] = $value;
            break;
          case 'Сумма':
            $data['inc_sum'] = $value;
            break;
          case 'Плательщик':
            $data['payer_name'] = $value;
            break;
          case 'ПлательщикИНН':
            $data['payer_inn'] = $value;
            break;
          case 'ПлательщикКПП':
            $data['payer_kpp'] = $value;
            break;
          case 'ПлательщикРасчСчет':
            $data['payer_schet'] = $value;
            break;
          case 'ПлательщикБанк1':
            $data['payer_bank'] = $value;
            break;
          case 'ПлательщикБИК':
            $data['payer_bik'] = $value;
            break;
          case 'ПлательщикКорсчет':
            $data['payer_kor_schet'] = $value;
            break;
          case 'Получатель':
            $data['recepient_name'] = $value;
            break;
          case 'ПолучательИНН':
            $data['recepient_inn'] = $value;
            break;
          case 'ПолучательКПП':
            $data['recepient_kpp'] = $value;
            break;
          case 'ПолучательРасчСчет':
            $data['recepient_schet'] = $value;
            break;
          case 'ПолучательБанк1':
            $data['recepient_bank'] = $value;
            break;
          case 'ПолучательБИК':
            $data['recepient_bik'] = $value;
            break;
          case 'ПолучательКорсчет':
            $data['recepient_kor_schet'] = $value;
            break;
          case 'НазначениеПлатежа':
            $data['destination_reason'] = $value;
            break;
          default:
            $data['other'] .= $param . ': ' . $value . PHP_EOL;
        }
      }

      $client_bank_type_doc = $this->getDocType($data['client_bank_type_doc']);
      if (!$client_bank_type_doc) {
        \Drupal::messenger()->addWarning(t('Unknown document type %doc_type.', ['%doc_type' => $data['client_bank_type_doc']]));
        continue;
      }
      $payment_contragent_data = $this->getPaymentContagentData($data);
      if (isset($payment_contragent_data['error'])) {
        \Drupal::messenger()->addError(t( $payment_contragent_data['error'] . ': %payer_name or %recepient_name', ['%payer_name' => $data['payer_name'], '%recepient_name' => $data['recepient_name']]));
        continue;
      }

      if ($payment_contragent_data['operation'] == 'outgoing') {
        $organization = $payment_contragent_data['payer_id'];
        $company = $payment_contragent_data['recepient_id'];
      }
      else {
        $organization = $payment_contragent_data['recepient_id'];
        $company = $payment_contragent_data['payer_id'];
      }

      $date = date('Y-m-d\T00:00:00', strtotime($data['document_date']));
      $query = \Drupal::entityQuery('node')
        ->accessCheck(TRUE)
        ->condition('type', 'trinion_payment_client_bank')
        ->condition('title', $data['nomer'])
        ->condition('field_tp_organizaciya', $organization)
        ->condition('field_tl_kompaniya', $company['contragent_nid'])
        ->condition('field_tcb_tip_dok_client_bank', $client_bank_type_doc)
        ->condition('field_tp_data', $date);
      $doc = $query->execute();
      if (empty($doc)) {
        $node = Node::create([
          'type' => 'trinion_payment_client_bank',
          'title' => $data['nomer'],
          'uid' => $uid,
          'status' => 1,
          'field_tcb_tip_dok_client_bank' => $client_bank_type_doc,
          'field_tp_otvetstvennyy' => $uid,
          'field_tp_data' => $date,
          'field_tcb_naznachenie_platezha' => $data['destination_reason'],
          'field_tp_organizaciya' => $organization,
          'field_tl_kompaniya' => $company['contragent_nid'],
          'field_tcb_prochie_polya' => $data['other'],
          'field_tp_itogo' => $data['inc_sum'],
        ]);
        $node->save();
        $count_created++;
      }
    }
    \Drupal::messenger()->addStatus(t('Created %count документов', ['%count' => $count_created]));
  }

  public function createContragent($data, $type) {
    $bundle = 'kompanii';
    $query = \Drupal::entityQuery('node')
      ->condition('type', $bundle)
      ->condition('field_tl_inn', $data[$type . '_inn']);
    $nid = $query->accessCheck()->execute();
    if (empty($nid)) {
      $uid = \Drupal::currentUser()->id();
      $node = Node::create([
        'type' => $bundle,
        'uid' => $uid,
        'status' => 1,
        'field_tl_otvetstvennyy' => $uid,
        'title' => $data[$type . '_name'],
        'field_tl_inn' => $data[$type . '_inn'],
        'field_tl_kpp' => $data[$type . '_kpp'],
      ]);
      $node->save();
      $contragent_nid = $node->id();
    }
    else
      $contragent_nid = reset($nid);
    $account_tid = $this->createBankAccount($data, $type, $contragent_nid);
    $contragent = Node::load($contragent_nid);
    $exist = FALSE;
    foreach ($contragent->get('field_tl_bankovskiy_schyot') as $schet) {
      if ($schet->getString() == $account_tid)
        $exist = TRUE;
    }
    if (!$exist) {
      $contragent->field_tl_bankovskiy_schyot[] = $account_tid;
      $contragent->save();
    }
    return [
      'contragent_nid' => $contragent_nid,
      'account_tid' => $account_tid,
    ];
  }

  public function createBankAccount($data, $type, $contragent_nid) {
    $vid = 'crm_bankovskie_scheta';
    $query = \Drupal::entityQuery('taxonomy_term')
      ->condition('vid', $vid)
      ->condition('field_tl_bik', $data[$type . '_bik'])
      ->condition('field_tl_rasschetnyy_schyot', $data[$type . '_schet']);
    $res = $query->accessCheck()->execute();
    if (empty($res)) {
      $term = Term::create([
        'vid' => $vid,
        'name' => $data[$type . '_schet'],
        'field_tl_bank' => $data[$type . '_bank'],
        'field_tl_bik' => $data[$type . '_bik'],
        'field_tl_kor_schyot' => $data[$type . '_kor_schet'],
        'field_tl_rasschetnyy_schyot' => $data[$type . '_schet'],
        'field_tl_kompaniya' => $contragent_nid,
      ]);
      $term->save();
      $account_tid = $term->id();
    }
    else
      $account_tid = reset($res);
    return $account_tid;
  }

  public function getDocType($name) {
    $doc_types = $this->getDocTypes();
    return array_search($name, $doc_types);
  }

  public function getDocTypes() {
    static $values = [];
    if (empty($values)) {
      $field_definitions = \Drupal::service('entity_field.manager')->getFieldDefinitions('node', 'trinion_payment_client_bank');
      if ($field_definitions) {
        $values = $field_definitions['field_tcb_tip_dok_client_bank']->getSetting('allowed_values');
      }
    }
    return $values;
  }

  public function getPaymentContagentData($data) {
    $result = [];
    if (empty($data['payer_inn'])) {
      \Drupal::messenger()->addError("У контрагента '{$data['payer_name']}' не заполнен ИНН. Документ {$data['nomer']}.");
    }
    elseif (empty($data['recepient_inn'])) {
      \Drupal::messenger()->addError("У контрагента '{$data['recepient_name']}' не заполнен ИНН. Документ {$data['nomer']}.");
    }
    else {
      $query = \Drupal::entityQuery('taxonomy_term')
        ->condition('field_tp_inn', $data['payer_inn']);
      $res = $query->accessCheck()->execute();
      if ($res) {
        $result['operation'] = 'outgoing';
        $result['payer_id'] = reset($res);
        $result['recepient_id'] = $this->createContragent($data, 'recepient');
      } else {
        $query = \Drupal::entityQuery('taxonomy_term')
          ->condition('field_tp_inn', $data['recepient_inn']);
        $res = $query->accessCheck()->execute();
        if ($res) {
          $result['operation'] = 'incoming';
          $result['recepient_id'] = reset($res);
          $result['payer_id'] = $this->createContragent($data, 'payer');
        } else {
          $result['error'] = 'Unknown organization';
        }
      }
    }
    return $result;
  }
}
